@@ -6,14 +6,26 @@ forecasts alongside observations.
 
 .. automodule:: forest.redux
 
+.. automodule:: forest.rx
+
+.. automodule:: forest.observe
+
+.. automodule:: forest.series
+
+.. automodule:: forest.colors
+
 .. automodule:: forest.keys
 
 .. automodule:: forest.config
 
 .. automodule:: forest.load
 
+.. automodule:: forest.geo
+
+.. automodule:: forest.presets
+
 """
-__version__ = '0.4.4'
+__version__ = '0.5.0'
 
 from .config import *
 from . import (
