@@ -21,6 +21,7 @@ applications.
 
 """
 import os
+import string
 import yaml
 from forest.export import export
 
@@ -28,6 +29,21 @@ from forest.export import export
 __all__ = []
 
 
+def combine_variables(os_environ, args_variables):
+    """Utility function to update environment with user-specified variables
+
+    .. note: When there is a key clash the user-specified args take precedence
+
+    :param os_environ: os.environ dict
+    :param args_variables: variables parsed from command line
+    :returns: merged dict
+    """
+    variables = dict(os_environ)
+    if args_variables is not None:
+        variables.update(dict(args_variables))
+    return variables
+
+
 class Config(object):
     """Configuration data structure
 
@@ -51,6 +67,24 @@ class Config(object):
                 self.__class__.__name__,
                 self.data)
 
+    @property
+    def presets_file(self):
+        """Colorbar presets JSON file
+
+        A location on disk where colorbar settings can be saved/loaded. If
+        the file does not exist it will be created by the application.
+
+        Use the following syntax to declare the presets file location
+
+        .. code-block:: yaml
+
+            presets:
+              file: ${HOME}/example/preset-save.json
+
+        :returns: location on disk to save colorbar presets
+        """
+        return self.data.get("presets", {}).get("file", None)
+
     @property
     def patterns(self):
         if "files" in self.data:
@@ -59,7 +93,7 @@ class Config(object):
         return []
 
     @classmethod
-    def load(cls, path):
+    def load(cls, path, variables=None):
         """Parse settings from either YAML or JSON file on disk
 
         The configuration can be controlled elegantly
@@ -73,25 +107,29 @@ class Config(object):
 
             files:
                 - label: Trial
-                  pattern: "*.nc"
-                  directory: trial/output
+                  pattern: "${TRIAL_DIR}/*.nc"
                 - label: Control
-                  pattern: "*.nc"
-                  directory: control/output
+                  pattern: "${CONTROL_DIR}/*.nc"
                 - label: RDT
-                  pattern: "*.json"
-                  directory: /satellite/rdt/json
+                  pattern: "${RDT_DIR}/*.json"
                   file_type: rdt
 
         :param path: JSON/YAML file to load
+        :param variables: dict of key/value pairs used by :py:class:`string.Template`
         :returns: instance of :class:`Config`
         """
         with open(path) as stream:
-            try:
-                # PyYaml 5.1 onwards
-                data = yaml.full_load(stream)
-            except AttributeError:
-                data = yaml.load(stream)
+            text = stream.read()
+
+        if variables is not None:
+            template = string.Template(text)
+            text = template.substitute(**variables)
+
+        try:
+            # PyYaml 5.1 onwards
+            data = yaml.safe_load(text)
+        except AttributeError:
+            data = yaml.load(text)
         return cls(data)
 
     @classmethod
@@ -120,6 +158,9 @@ class FileGroup(object):
     contained within the files or how data is catalogued
     and searched.
 
+    .. note:: This class violates the integration separation principle (ISP)
+              all driver settings are included in the same constructor
+
     :param label: decription used by buttons and tooltips
     :param pattern: wildcard pattern used by either SQL or glob
     :param locator: keyword describing search method (default: 'file_system')
@@ -131,12 +172,14 @@ class FileGroup(object):
             pattern,
             locator="file_system",
             file_type="unified_model",
-            directory=None):
+            directory=None,
+            database_path=None):
         self.label = label
         self.pattern = pattern
         self.locator = locator
         self.file_type = file_type
         self.directory = directory
+        self.database_path = database_path
 
     @property
     def full_pattern(self):
@@ -161,7 +204,8 @@ class FileGroup(object):
         kwarg_attrs = [
             "locator",
             "file_type",
-            "directory"]
+            "directory",
+            "database_path"]
         kwargs = [
             "{}={}".format(attr, self._str(getattr(self, attr)))
                 for attr in kwarg_attrs]
